// Skeleton Program code for the AQA A Level Paper 1 Summer 2022 examination
// this code should be used in conjunction with the Preliminary Material
// written by the AQA Programmer Team
// developed in the Lazarus programming environment

program Pascal_A2_2022;

{$APPTYPE CONSOLE}
{$MODE DELPHI}  // Delete this line if using the Delphi programming environment

uses
  Classes,
  SysUtils,
  Math,
  Generics.Collections;

type
  Challenge = class
    protected
      Condition : TArray<string>;
      Met : boolean;
    public
      constructor New();
      function GetMet() : boolean;
      function GetCondition() : TArray<string>;
      procedure SetMet(NewValue : boolean);
      procedure SetCondition(NewCondition : TArray<string>);
  end;

  Lock = class
    protected
      Challenges : TList<Challenge>;
    private
      function ConvertConditionToString(C : TArray<string>) : string;
    public
      constructor New();
      procedure AddChallenge(Condition : TArray<string>); virtual;
      function GetLockDetails() : string;
      function GetLockSolved() : boolean; virtual;
      function CheckIfConditionMet(Sequence : string) : boolean; virtual;
      procedure SetChallengeMet(Pos : integer; Value : boolean);
      function GetChallengeMet(Pos : integer) : boolean; virtual;
      function GetNumberOfChallenges() : integer; virtual;
  end;

  CardCollection = class;

  Card = class
    private
      class var NextCardNumber: integer;
    protected
      CardNumber, Score : integer;
    public
      constructor New();
      function GetScore() : integer; virtual;
      procedure Process(Deck : CardCollection; Discard : CardCollection; Hand : CardCollection; Sequence : CardCollection; CurrentLock : Lock; Choice : string; CardChoice : integer); virtual;
      function GetCardNumber() : integer; virtual;
      function GetDescription() : string; virtual;
  end;

  ToolCard = class(Card)
    protected
      ToolType : string;
      Kit : string;
    private
      procedure SetScore();
    public
      constructor New(T : string; K : string); overload;
      constructor New(T : string; K : string; CardNo : integer); overload;
      function GetDescription() : string; override;
  end;

  DifficultyCard = class(Card)
    protected
      CardType : string;
    public
      constructor New(); overload;
      constructor New(CardNo : integer); overload;
      function GetDescription() : string; override;
      procedure Process(Deck : CardCollection; Discard : CardCollection; Hand : CardCollection; Sequence : CardCollection; CurrentLock : Lock; Choice : string; CardChoice : integer); override;
    end;

  CardCollection = class
    protected
      Cards : TList<Card>;
      Name : string;
    public
      constructor New(N : string);
      function GetName() : string;
      function GetCardNumberAt(X : integer) : integer;
      function GetCardDescriptionAt(X : integer) : string;
      procedure AddCard(C : Card);
      function GetNumberOfCards() : integer;
      procedure Shuffle();
      function RemoveCard(CardNumber : integer) : Card;
      function GetCardDisplay() : string;
    private
      function CreateLineOfDashes(Size : integer) : string;
  end;

  Breakthrough = class
    private
      Deck, Hand, Sequence, Discard : CardCollection;
      Locks : TList<Lock>;
      Score : integer;
      GameOver, LockSolved : Boolean;
      CurrentLock : Lock;
      procedure ProcessLockSolved();
      function CheckIfPlayerHasLost() : boolean;
      procedure SetupGame();
      procedure PlayCardToSequence(CardChoice : integer);
      function CheckIfLockChallengeMet() : boolean;
      procedure SetupCardCollectionFromGameFile(LineFromFile : string; CardCol : CardCollection);
      procedure SetupLock(LineOne : string; LineTwo : string);
      function LoadGame(FileName : string) : boolean;
      procedure LoadLocks();
      function GetRandomLock() : Lock;
      procedure GetCardFromDeck(CardChoice : integer);
      function GetCardChoice() : integer;
      function GetDiscardOrPlayChoice() : string;
      function GetChoice() : string;
      procedure AddDifficultyCardsToDeck();
      procedure CreateStandardDeck();
      function MoveCard(FromCollection : CardCollection; ToCollection : CardCollection; CardNumber : integer) : integer;
    public
      constructor New();
      procedure PlayGame();
  end;

  constructor Challenge.New();
  begin
    Met := False;
  end;

  function Challenge.GetMet() : boolean;
  begin
    result := Met;
  end;

  function Challenge.GetCondition() : TArray<string>;
  begin
    result := Condition;
  end;

  procedure Challenge.SetMet(NewValue : boolean);
  begin
    Met := NewValue;
  end;

  procedure Challenge.SetCondition(NewCondition : TArray<string>);
  begin
    Condition := NewCondition;
  end;

  constructor Lock.New();
  begin
    Challenges := TList<Challenge>.Create();
  end;

  function Lock.ConvertConditionToString(C : TArray<string>) : string;
  var
    ConditionAsString : string;
    Pos : integer;
  begin
    ConditionAsString := '';
    for Pos := 0 to length(C) - 2 do
    begin
      ConditionAsString := ConditionAsString + C[Pos] + ', ';
    end;
    ConditionAsString := ConditionAsString + C[length(C) - 1];
    result := ConditionAsString;
  end;

  procedure Lock.AddChallenge(Condition : TArray<string>);
  var
    C : Challenge;
  begin
    C := Challenge.New();
    C.SetCondition(Condition);
    Challenges.Add(C);
  end;

  function Lock.GetLockDetails() : string;
  var
    LockDetails : string;
    c : Challenge;
  begin
    LockDetails := sLineBreak + 'CURRENT LOCK' + sLineBreak + '------------' + sLineBreak;
    for c in Challenges do
    begin
      if c.GetMet() then
      begin
        LockDetails := LockDetails + 'Challenge met: ';
      end
      else
      begin
        LockDetails := LockDetails + 'Not met:       ';
      end;
      LockDetails := LockDetails + ConvertConditionToString(c.GetCondition()) + sLineBreak;
    end;
    LockDetails := LockDetails + sLineBreak;
    result := LockDetails;
  end;

  function Lock.GetLockSolved() : boolean;
  var
    c : Challenge;
  begin
    result := True;
    for c in Challenges do
    begin
      if not c.GetMet() then
      begin
        result := False;
      end;
    end;
  end;

  function Lock.CheckIfConditionMet(Sequence : string) : boolean;
  var
    c : Challenge;
  begin
    result := False;
    for c in Challenges do
    begin
      if ((not c.GetMet()) and (Sequence = ConvertConditionToString(c.GetCondition()))) then
      begin
        c.SetMet(True);
        result := True;
      end;
    end;
  end;

  procedure Lock.SetChallengeMet(Pos : integer; Value : boolean);
  begin
    Challenges[Pos].SetMet(Value);
  end;

  function Lock.GetChallengeMet(Pos : integer) : boolean;
  begin
    result := Challenges.Items[pos].GetMet();
  end;

  function Lock.GetNumberOfChallenges() : integer;
  begin
    result := Challenges.Count;
  end;

  constructor Card.New();
  begin
    CardNumber := NextCardNumber;
    inc(NextCardNumber);
    Score := 0;
  end;

  function Card.GetScore() : integer;
  begin
    result := Score;
  end;

  procedure Card.Process(Deck : CardCollection; Discard : CardCollection; Hand : CardCollection; Sequence : CardCollection; CurrentLock : Lock; Choice : string; CardChoice : integer);
  begin
  end;

  function Card.GetCardNumber() : integer;
  begin
    result := CardNumber;
  end;

  function Card.GetDescription() : string;
  begin
    if CardNumber < 10 then
    begin
      result := ' ' + IntToStr(CardNumber)
    end
    else
    begin
      result := IntToStr(CardNumber);
    end;
  end;

  constructor ToolCard.New(T : string; K : string);
  begin
    self.New();
    ToolType := T;
    Kit := K;
    SetScore();
  end;

  constructor ToolCard.New(T : string; K : string; CardNo : integer);
  begin
    ToolType := T;
    Kit := K;
    CardNumber := CardNo;
    SetScore();
  end;

  procedure ToolCard.SetScore();
  begin
    Case ToolType[1] of
      'K' : Score := 3;
      'F' : Score := 2;
      'P' : Score := 1;
    end;
  end;

  function ToolCard.GetDescription() : string;
  begin
    result := ToolType + ' ' + Kit;
  end;

  constructor DifficultyCard.New();
  begin
    inherited;
    CardType := 'Dif';
  end;

  constructor DifficultyCard.New(CardNo : integer);
  begin
    CardType := 'Dif';
    CardNumber := CardNo;
  end;

  function DifficultyCard.GetDescription() : string;
  begin
    result := CardType;
  end;

  procedure DifficultyCard.Process(Deck : CardCollection; Discard : CardCollection; Hand : CardCollection; Sequence : CardCollection; CurrentLock : Lock; Choice : string; CardChoice : integer);
  var
    ChoiceAsInteger, Count : integer;
    CardToMove : Card;
  begin
    try
      ChoiceAsInteger := StrToInt(Choice);
      if (ChoiceAsInteger >= 1) and (ChoiceAsInteger <= 5) then
      begin
        if ChoiceAsInteger >= CardChoice then
        begin
          dec(ChoiceAsInteger);
        end;
        if ChoiceAsInteger > 0 then
        begin
          dec(ChoiceAsInteger);
        end;
        if Hand.GetCardDescriptionAt(ChoiceAsInteger)[1] = 'K' then
        begin
          CardToMove := Hand.RemoveCard(Hand.GetCardNumberAt(ChoiceAsInteger));
          Discard.AddCard(CardToMove);
          exit;
        end;
      end;
    except
    end;
    Count := 0;
    while (Count < 5) and (Deck.GetNumberOfCards() > 0) do
    begin
      CardToMove := Deck.RemoveCard(Deck.GetCardNumberAt(0));
      Discard.AddCard(CardToMove);
      inc(Count);
    end;
  end;

  constructor CardCollection.New(N : string);
  begin
    Name := N;
    Cards := TList<Card>.Create();
  end;

  function CardCollection.GetName() : string;
  begin
    result := Name;
  end;

  function CardCollection.GetCardNumberAt(X : integer) : integer;
  begin
    result := Cards[X].GetCardNumber();
  end;

  function CardCollection.GetCardDescriptionAt(X : integer) : string;
  begin
    result := Cards[X].GetDescription();
  end;

  procedure CardCollection.AddCard(C : Card);
  begin
    Cards.Add(C);
  end;

  function CardCollection.GetNumberOfCards() : integer;
  begin
    result := Cards.Count;
  end;

  procedure CardCollection.Shuffle();
  var
    TempCard : Card;
    RNo1, RNo2, Count : integer;
  begin
    randomize;
    for Count := 1 to 1000 do
    begin
      RNo1 := random(Cards.Count);
      RNo2 := random(Cards.Count);
      TempCard := Cards[RNo1];
      Cards[RNo1] := Cards[RNo2];
      Cards[RNo2] := TempCard;
    end;
  end;

  function CardCollection.RemoveCard(CardNumber : integer) : Card;
  var
    CardFound : boolean;
    Pos : integer;
    CardToGet : Card;
  begin
    CardFound := False;
    Pos := 0;
    CardToGet := nil;
    while (Pos < Cards.Count) and (not CardFound) do
    begin
      if Cards[Pos].GetCardNumber() = CardNumber then
      begin
        CardToGet := Cards[Pos];
        CardFound := True;
        Cards.Delete(Pos);
      end;
      inc(Pos);
    end;
    result := CardToGet;
  end;

  function CardCollection.CreateLineOfDashes(Size : integer) : string;
  var
    Count : integer;
  begin
    result := '';
    for Count := 1 to Size do
    begin
      result := result + '------';
    end;
  end;

  function CardCollection.GetCardDisplay() : string;
  const
    CardsPerLine : integer = 10;
  var
    CardDisplay, LineOfDashes : String;
    Complete : boolean;
    Pos : integer;
  begin
    CardDisplay := sLineBreak + Name + ':';
    if Cards.Count = 0 then
    begin
      GetCardDisplay := CardDisplay + ' empty' + sLineBreak + sLineBreak;
      exit;
    end
    else
    begin
      CardDisplay := CardDisplay + sLineBreak + sLineBreak;
    end;
    LineOfDashes := '';
    if Cards.Count > CardsPerLine then
    begin
      LineOfDashes := CreateLineOfDashes(CardsPerLine)
    end
    else
    begin
      LineOfDashes := CreateLineOfDashes(Cards.Count);
    end;
    CardDisplay := CardDisplay + LineOfDashes + sLineBreak;
    Complete := False;
    Pos := 0;
    while not Complete do
    begin
      CardDisplay := CardDisplay + '| ' + Cards[Pos].GetDescription() + ' ';
      inc(Pos);
      if Pos mod CardsPerLine = 0 then
      begin
        CardDisplay := CardDisplay + '|' + sLineBreak + LineOfDashes + sLineBreak;
      end;
      if Pos = Cards.Count then
      begin
        Complete := True;
      end;
    end;
    if (Cards.Count mod CardsPerLine) > 0 then
    begin
      CardDisplay := CardDisplay + '|' + sLineBreak;
      if Cards.Count > CardsPerLine then
      begin
        LineOfDashes := CreateLineOfDashes(Cards.Count mod CardsPerLine);
      end;
      CardDisplay := CardDisplay + LineOfDashes + sLineBreak;
    end;
    result := CardDisplay;
  end;

  constructor Breakthrough.New();
  begin
    Deck := CardCollection.New('DECK');
    Hand := CardCollection.New('HAND');
    Sequence := CardCollection.New('SEQUENCE');
    Discard := CardCollection.New('DISCARD');
    Locks := TList<Lock>.Create();
    Score := 0;
    LoadLocks();
  end;

  procedure Breakthrough.ProcessLockSolved();
  begin
    Score := Score + 10;
    writeln('Lock has been solved.  Your score is now: ' + IntToStr(Score));
    while Discard.GetNumberOfCards() > 0 do
    begin
      MoveCard(Discard, Deck, Discard.GetCardNumberAt(0));
    end;
    Deck.Shuffle();
    CurrentLock := GetRandomLock();
  end;

  function Breakthrough.CheckIfPlayerHasLost() : boolean;
  begin
    if Deck.GetNumberOfCards() = 0 then
    begin
      writeln('You have run out of cards in your deck.  Your final score is: ' + inttostr(Score));
      result := True;
    end
    else
    begin
      result := False;
    end;
  end;

  procedure Breakthrough.SetupGame();
  var
    Choice : string;
    Count : integer;
  begin
    write('Enter L to load a game from a file, anything else to play a new game:> ');
    readln(Choice);
    Choice := UpperCase(Choice);
    if Choice = 'L' then
      begin
        if not LoadGame('game1.txt') then
        begin
          GameOver := True
        end;
      end
    else
    begin
      CreateStandardDeck();
      Deck.Shuffle();
      for Count := 1 to 5 do
      begin
        MoveCard(Deck, Hand, Deck.GetCardNumberAt(0));
      end;
      AddDifficultyCardsToDeck();
      Deck.Shuffle();
      CurrentLock := GetRandomLock();
    end;
  end;

  procedure Breakthrough.PlayCardToSequence(CardChoice : integer);
  begin
    if Sequence.GetNumberOfCards() > 0 then
      begin
        if Hand.GetCardDescriptionAt(CardChoice - 1)[1] <> Sequence.GetCardDescriptionAt(Sequence.GetNumberOfCards() - 1)[1] then
        begin
          Score := Score + MoveCard(Hand, Sequence, Hand.GetCardNumberAt(CardChoice - 1));
          GetCardFromDeck(CardChoice);
        end
      end
    else
    begin
      Score := Score + MoveCard(Hand, Sequence, Hand.GetCardNumberAt(CardChoice - 1));
      GetCardFromDeck(CardChoice);
    end;
    if CheckIfLockChallengeMet() then
    begin
      writeln;
      writeln('A challenge on the lock has been met.');
      writeln;
      Score := Score + 5;
    end;
  end;

  function Breakthrough.CheckIfLockChallengeMet() : boolean;
  var
    SequenceAsString : string;
    Count : integer;
  begin
    SequenceAsString := '';
    for Count := Sequence.GetNumberOfCards() - 1 downto Max(0, Sequence.GetNumberOfCards() - 3) do
    begin
      if length(SequenceAsString) > 0 then
      begin
        SequenceAsString := ', ' + SequenceAsString;
      end;
      SequenceAsString := Sequence.GetCardDescriptionAt(Count) + SequenceAsString;
      if CurrentLock.CheckIfConditionMet(SequenceAsString) then
      begin
        result := True;
        exit;
      end;
    end;
    result := False;
  end;

  procedure Breakthrough.SetupCardCollectionFromGameFile(LineFromFile : string; CardCol : CardCollection);
  var
    SplitLine : TArray<string>;
    CardNumber, Index : integer;
    Item : String;
    CurrentCard : Card;
  begin
    if length(LineFromFile) > 0 then
    begin
      SplitLine := LineFromFile.Split([',']);
      for Index := 0 to length(SplitLine) - 1 do
      begin
        Item := SplitLine[Index];
        if length(Item) = 5 then
        begin
          CardNumber := strtoint(Item[5])
        end
        else
        begin
          CardNumber := strtoint(copy(Item, 5, 2));
        end;
        if copy(Item, 1, 3) = 'Dif' then
        begin
          CurrentCard := DifficultyCard.New(CardNumber);
          CardCol.AddCard(CurrentCard);
        end
        else
        begin
          CurrentCard := ToolCard.New(Item[1], Item[3], CardNumber);
          CardCol.AddCard(CurrentCard);
        end;
      end;
    end;
  end;

  procedure Breakthrough.SetupLock(LineOne : string; LineTwo : string);
  var
    SplitLine, Conditions : TArray<string>;
    Item : string;
    Count, Index : integer;
  begin
    SplitLine := LineOne.Split([';']);
    for Index := 0 to length(SplitLine) - 1 do
    begin
      Item := SplitLine[Index];
      Conditions := Item.Split([',']);
      CurrentLock.AddChallenge(Conditions);
    end;
    SplitLine := LineTwo.split([';']);
    for Count := 0 to length(SplitLine) - 1 do
    begin
      if SplitLine[Count] = 'Y' then
      begin
        CurrentLock.SetChallengeMet(Count, True);
      end;
    end;
  end;

  function Breakthrough.LoadGame(FileName : string) : boolean;
  var
    LineFromFile, LineFromFile2 : string;
    TextFile : text;
  begin
    try
      AssignFile(TextFile, FileName);
      Reset(TextFile);
      readln(TextFile, LineFromFile);
      Score := strtoint(LineFromFile);
      readln(TextFile, LineFromFile);
      readln(TextFile, LineFromFile2);
      SetupLock(LineFromFile, LineFromFile2);
      readln(TextFile, LineFromFile);
      SetupCardCollectionFromGameFile(LineFromFile, Hand);
      readln(TextFile, LineFromFile);
      SetupCardCollectionFromGameFile(LineFromFile, Sequence);
      readln(TextFile, LineFromFile);
      SetupCardCollectionFromGameFile(LineFromFile, Discard);
      readln(TextFile, LineFromFile);
      SetupCardCollectionFromGameFile(LineFromFile, Deck);
      result := True;
    except
      writeln('File not loaded');
      result := False;
    end;
  end;

  procedure Breakthrough.LoadLocks();
  var
    FileName, LineFromFile, C : string;
    Challenges, Conditions : TArray<string>;
    LockFromFile : Lock;
    TextFile : text;
  begin
    FileName := 'locks.txt';
    try
      AssignFile(TextFile, FileName);
      Reset(TextFile);
      readln(TextFile, LineFromFile);
      while not eof(TextFile) do
      begin
        Challenges := LineFromFile.Split([';']);
        LockFromFile := Lock.New();
        for c in Challenges do
        begin
          Conditions := C.Split([',']);
          LockFromFile.AddChallenge(Conditions);
        end;
        Locks.Add(LockFromFile);
        readln(TextFile, LineFromFile);
      end;
    except
      writeln('File not loaded');
    end;
  end;

  function Breakthrough.GetRandomLock() : Lock;
  begin
    result := Locks[Random(Locks.Count)];
  end;

  procedure Breakthrough.GetCardFromDeck(CardChoice : integer);
  var
    CurrentCard : Card;
    Choice : string;
  begin
    if Deck.GetNumberOfCards() > 0 then
    begin
      if Deck.GetCardDescriptionAt(0) = 'Dif' then
      begin
        CurrentCard := Deck.RemoveCard(Deck.GetCardNumberAt(0));
        writeln;
        writeln('Difficulty encountered!');
        writeln(Hand.GetCardDisplay());
        write('To deal with this you need to either lose a key ');
        write('(enter 1-5 to specify position of key) or (D)iscard five cards from the deck:> ');
        readln(Choice);
        writeln;
        Discard.AddCard(CurrentCard);
        CurrentCard.Process(Deck, Discard, Hand, Sequence, CurrentLock, Choice, CardChoice);
      end;
    end;
    while (Hand.GetNumberOfCards() < 5) and (Deck.GetNumberOfCards() > 0) do
    begin
      if Deck.GetCardDescriptionAt(0) = 'Dif' then
      begin
        MoveCard(Deck, Discard, Deck.GetCardNumberAt(0));
        writeln('A difficulty card was discarded from the deck when refilling the hand.');
      end
      else
      begin
        MoveCard(Deck, Hand, Deck.GetCardNumberAt(0));
      end;
    end;
    if (Deck.GetNumberOfCards() = 0) and (Hand.GetNumberOfCards() < 5) then
    begin
      GameOver := True;
    end;
  end;

  function Breakthrough.GetCardChoice() : integer;
  var
    Choice : string;
    Converted : boolean;
  begin
    repeat
      Converted := False;
      write('Enter a number between 1 and 5 to specify card to use:> ');
      readln(Choice);
      try
        result := strtoint(Choice);
        Converted := True;
      except
      end;
    until Converted;
  end;

  function Breakthrough.GetDiscardOrPlayChoice() : string;
  var
    Choice : string;
  begin
    write('(D)iscard or (P)lay?:> ');
    readln(Choice);
    Choice := UpperCase(Choice);
    result := Choice;
  end;

  function Breakthrough.GetChoice() : string;
  var
    Choice : string;
  begin
    writeln;
    write('(D)iscard inspect, (U)se card:> ');
    readln(Choice);
    Choice := UpperCase(Choice);
    result := Choice;
  end;

  procedure Breakthrough.AddDifficultyCardsToDeck();
  var
    Count : integer;
  begin
    for Count := 1 to 5 do
    begin
      Deck.AddCard(DifficultyCard.New());
    end;
  end;

  procedure Breakthrough.CreateStandardDeck();
  var
    NewCard : Card;
    Count : integer;
  begin
    for Count := 1 to 5 do
    begin
      NewCard := ToolCard.New('P', 'a');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('P', 'b');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('P', 'c');
      Deck.AddCard(NewCard);
    end;
    for Count := 1 to 3 do
    begin
      NewCard := ToolCard.New('F', 'a');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('F', 'b');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('F', 'c');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('K', 'a');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('K', 'b');
      Deck.AddCard(NewCard);
      NewCard := ToolCard.New('K', 'c');
      Deck.AddCard(NewCard);
    end;
  end;

  function Breakthrough.MoveCard(FromCollection : CardCollection; ToCollection : CardCollection; CardNumber : integer) : integer;
  var
    CardScore : integer;
    CardToMove : Card;
  begin
    CardScore := 0;
    if (FromCollection.GetName() = 'HAND') and (ToCollection.GetName() = 'SEQUENCE') then
    begin
      CardToMove := FromCollection.RemoveCard(CardNumber);
      if CardToMove <> nil then
      begin
        ToCollection.AddCard(CardToMove);
        CardScore := CardToMove.GetScore();
      end;
    end
    else
    begin
      CardToMove := FromCollection.RemoveCard(CardNumber);
      if CardToMove <> nil then
      begin
        ToCollection.AddCard(CardToMove);
      end;
    end;
    result := CardScore;
  end;

  procedure Breakthrough.PlayGame();
  var
    MenuChoice, DiscardOrPlay : string;
    CardChoice : integer;
  begin
    if Locks.Count > 0 then
    begin
      GameOver := False;
      CurrentLock := Lock.New();
      SetupGame();
      while not GameOver do
      begin
        LockSolved := False;
        while (not LockSolved) and (not GameOver) do
        begin
          writeln;
          writeln('Current score: ' + IntToStr(Score));
          writeln(CurrentLock.GetLockDetails());
          writeln(Sequence.GetCardDisplay());
          writeln(Hand.GetCardDisplay());
          MenuChoice := GetChoice();
          case MenuChoice[1] of
            'D' : begin
                    writeln(Discard.GetCardDisplay());
                  end;

            'U' : begin
                    CardChoice := GetCardChoice();
                    DiscardOrPlay := GetDiscardOrPlayChoice();
                    if DiscardOrPlay = 'D' then
                    begin
                      MoveCard(Hand, Discard,Hand.GetCardNumberAt(CardChoice - 1));
                      GetCardFromDeck(CardChoice);
                    end
                    else if DiscardOrPlay = 'P' then
                    begin
                      PlayCardToSequence(CardChoice);
                    end;
                  end;
          end;
          if CurrentLock.GetLockSolved() then
          begin
            LockSolved := True;
            ProcessLockSolved();
          end;
        end;
        GameOver := CheckIfPlayerHasLost();
      end;
    end
    else
    begin
      writeln('No locks in file.');
    end;
  end;

var
  ThisGame : Breakthrough;

begin
  ThisGame := Breakthrough.New();
  ThisGame.PlayGame();
  readln;
end.
